#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from elasticsearch_dsl import Q
from zope.intid.interfaces import IIntIds

from pyams_content.shared.view.interfaces import IViewInternalReferencesSettings, IViewQueryEsParamsExtension, \
    IWfView, ONLY_REFERENCE_MODE
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.registry import get_utility


@adapter_config(name='references', context=IWfView, provides=IViewQueryEsParamsExtension)
class ViewInternalReferencesQueryEsParamsExtension(ContextAdapter):
    """View internal references query params extension"""

    weight = 10

    def get_es_params(self, context, request=None):
        settings = IViewInternalReferencesSettings(self.context)
        # check view references mode
        if settings.references_mode == ONLY_REFERENCE_MODE:
            # references are retrieved by query filter extension, so no params are required!
            yield None
        else:
            # check view settings
            if settings.exclude_context:
                intids = get_utility(IIntIds)
                try:
                    yield Q('bool',
                            must_not=Q('term', **{'internal_id': intids.register(context)}))
                except TypeError:
                    return
